@@ -5,8 +5,10 @@
 #
 # License: BSD (3-clause)
 import os.path as op
-import pytest
+import datetime
+from distutils.version import LooseVersion
 
+import pytest
 from scipy.io import savemat
 
 # This is here to handle mne-python <0.20
@@ -19,15 +21,17 @@ with warnings.catch_warnings():
 
 from mne.datasets import testing
 from mne.utils import _TempDir
-from mne_bids.utils import _handle_kind, _parse_ext
+from mne_bids import BIDSPath
+from mne_bids.utils import _handle_datatype
+from mne_bids.path import _parse_ext
 
 from mne_bids.copyfiles import (_get_brainvision_encoding,
                                 _get_brainvision_paths,
                                 copyfile_brainvision,
+                                copyfile_edf,
                                 copyfile_eeglab,
                                 copyfile_kit)
 
-from mne_bids.write import make_bids_basename
 
 base_path = op.join(op.dirname(mne.__file__), 'io')
 
@@ -76,6 +80,8 @@ def test_get_brainvision_paths():
     assert tail == 'test.vmrk'
 
 
+@pytest.mark.filterwarnings('ignore:.*Exception ignored.*:'
+                            'pytest.PytestUnraisableExceptionWarning')
 def test_copyfile_brainvision():
     """Test the copying of BrainVision vhdr, vmrk and eeg files."""
     bids_root = _TempDir()
@@ -100,12 +106,95 @@ def test_copyfile_brainvision():
     raw = mne.io.read_raw_brainvision(new_name)
     assert raw.filenames[0] == (op.join(head, 'tested_conversion.eeg'))
 
+    # Test with anonymization
+    raw = mne.io.read_raw_brainvision(raw_fname)
+    prev_date = raw.info['meas_date']
+    anonymize = {'daysback': 32459}
+    copyfile_brainvision(raw_fname, new_name, anonymize, verbose=True)
+    raw = mne.io.read_raw_brainvision(new_name)
+    new_date = raw.info['meas_date']
+    assert new_date == (prev_date - datetime.timedelta(days=32459))
 
-def test_copyfile_eeglab():
+
+def test_copyfile_edf():
+    """Test the anonymization of EDF/BDF files"""
+    bids_root = _TempDir()
+    data_path = op.join(base_path, 'edf', 'tests', 'data')
+
+    # Test regular copying
+    for ext in ['.edf', '.bdf']:
+        raw_fname = op.join(data_path, 'test' + ext)
+        new_name = op.join(bids_root, 'test_copy' + ext)
+        copyfile_edf(raw_fname, new_name)
+
+    # IO error testing
+    with pytest.raises(ValueError, match='Need to move data with same'):
+        raw_fname = op.join(data_path, 'test.edf')
+        new_name = op.join(bids_root, 'test_copy.bdf')
+        copyfile_edf(raw_fname, new_name)
+
+    # Add some subject info to an EDF to test anonymization
+    testfile = op.join(bids_root, 'test_copy.edf')
+    raw_date = mne.io.read_raw_edf(testfile).info['meas_date']
+    date = datetime.datetime.strftime(raw_date, "%d-%b-%Y").upper()
+    test_id_info = '023 F 02-AUG-1951 Jane'
+    test_rec_info = 'Startdate {0} ID-123 John BioSemi_ActiveTwo'.format(date)
+    with open(testfile, 'r+b') as f:
+        f.seek(8)
+        f.write(bytes(test_id_info.ljust(80), 'ascii'))
+        f.write(bytes(test_rec_info.ljust(80), 'ascii'))
+
+    # Test date anonymization
+    def _edf_get_real_date(fpath):
+        with open(fpath, 'rb') as f:
+            f.seek(88)
+            rec_info = f.read(80).decode('ascii').rstrip()
+        startdate = rec_info.split(' ')[1]
+        return datetime.datetime.strptime(startdate, "%d-%b-%Y")
+
+    bids_root2 = _TempDir()
+    infile = op.join(bids_root, 'test_copy.edf')
+    outfile = op.join(bids_root2, 'test_copy_anon.edf')
+    anonymize = {'daysback': 33459, 'keep_his': False}
+    copyfile_edf(infile, outfile, anonymize)
+    prev_date = _edf_get_real_date(infile)
+    new_date = _edf_get_real_date(outfile)
+    assert new_date == (prev_date - datetime.timedelta(days=33459))
+
+    # Test full ID info anonymization
+    anon_startdate = datetime.datetime.strftime(new_date, "%d-%b-%Y").upper()
+    with open(outfile, 'rb') as f:
+        f.seek(8)
+        id_info = f.read(80).decode('ascii').rstrip()
+        rec_info = f.read(80).decode('ascii').rstrip()
+    rec_info_tmp = "Startdate {0} X mne-bids_anonymize X"
+    assert id_info == "0 X X X"
+    assert rec_info == rec_info_tmp.format(anon_startdate)
+
+    # Test partial ID info anonymization
+    outfile2 = op.join(bids_root2, 'test_copy_anon_partial.edf')
+    anonymize = {'daysback': 33459, 'keep_his': True}
+    copyfile_edf(infile, outfile2, anonymize)
+    with open(outfile2, 'rb') as f:
+        f.seek(8)
+        id_info = f.read(80).decode('ascii').rstrip()
+        rec_info = f.read(80).decode('ascii').rstrip()
+    rec = 'Startdate {0} ID-123 John BioSemi_ActiveTwo'.format(anon_startdate)
+    assert id_info == "023 F X X"
+    assert rec_info == rec
+
+
+@pytest.mark.parametrize('fname',
+                         ('test_raw.set', 'test_raw_chanloc.set'))
+def test_copyfile_eeglab(fname):
     """Test the copying of EEGlab set and fdt files."""
+    if (fname == 'test_raw_chanloc.set' and
+            LooseVersion(testing.get_version()) < LooseVersion('0.112')):
+        return
+
     bids_root = _TempDir()
     data_path = op.join(testing.data_path(), 'EEGLAB')
-    raw_fname = op.join(data_path, 'test_raw.set')
+    raw_fname = op.join(data_path, fname)
     new_name = op.join(bids_root, 'tested_conversion.set')
 
     # IO error testing
@@ -121,7 +210,14 @@ def test_copyfile_eeglab():
 
     # Test copying and reading a combined set+fdt
     copyfile_eeglab(raw_fname, new_name)
-    raw = mne.io.read_raw_eeglab(new_name)
+    if fname == 'test_raw_chanloc.set':
+        with pytest.warns(RuntimeWarning,
+                          match="The data contains 'boundary' events"):
+            raw = mne.io.read_raw_eeglab(new_name)
+            assert 'Fp1' in raw.ch_names
+    else:
+        raw = mne.io.read_raw_eeglab(new_name)
+        assert 'EEG 001' in raw.ch_names
     assert isinstance(raw, mne.io.BaseRaw)
 
 
@@ -139,45 +235,48 @@ def test_copyfile_kit():
     acq = '01'
     task = 'testing'
 
-    bids_basename = make_bids_basename(
-        subject=subject_id, session=session_id, run=run, acquisition=acq,
-        task=task)
-
-    kit_bids_basename = bids_basename.replace('_acq-01', '')
-
     raw = mne.io.read_raw_kit(
         raw_fname, mrk=hpi_fname, elp=electrode_fname,
         hsp=headshape_fname)
     _, ext = _parse_ext(raw_fname, verbose=True)
-    kind = _handle_kind(raw)
-    bids_fname = bids_basename + '_%s%s' % (kind, ext)
-    bids_fname = op.join(output_path, bids_fname)
+    datatype = _handle_datatype(raw)
+
+    bids_path = BIDSPath(
+        subject=subject_id, session=session_id, run=run, acquisition=acq,
+        task=task)
+    kit_bids_path = bids_path.copy().update(acquisition=None,
+                                            datatype=datatype,
+                                            root=output_path)
+    bids_fname = str(bids_path.copy().update(datatype=datatype,
+                                             suffix=datatype,
+                                             extension=ext,
+                                             root=output_path))
 
     copyfile_kit(raw_fname, bids_fname, subject_id, session_id,
                  task, run, raw._init_kwargs)
     assert op.exists(bids_fname)
     _, ext = _parse_ext(hpi_fname, verbose=True)
     if ext == '.sqd':
-        assert op.exists(op.join(
-            output_path, kit_bids_basename + '_markers.sqd'))
+        kit_bids_path.update(suffix='markers', extension='.sqd')
+        assert op.exists(kit_bids_path)
     elif ext == '.mrk':
-        assert op.exists(op.join(
-            output_path, kit_bids_basename + '_markers.mrk'))
+        kit_bids_path.update(suffix='markers', extension='.mrk')
+        assert op.exists(kit_bids_path)
 
     if op.exists(electrode_fname):
         task, run, key = None, None, 'ELP'
         elp_ext = '.pos'
-        elp_fname = make_bids_basename(
+        elp_fname = BIDSPath(
             subject=subject_id, session=session_id, task=task, run=run,
-            acquisition=key, suffix='headshape%s' % elp_ext,
-            prefix=output_path)
+            acquisition=key, suffix='headshape', extension=elp_ext,
+            datatype='meg', root=output_path)
         assert op.exists(elp_fname)
 
     if op.exists(headshape_fname):
         task, run, key = None, None, 'HSP'
         hsp_ext = '.pos'
-        hsp_fname = make_bids_basename(
+        hsp_fname = BIDSPath(
             subject=subject_id, session=session_id, task=task, run=run,
-            acquisition=key, suffix='headshape%s' % hsp_ext,
-            prefix=output_path)
+            acquisition=key, suffix='headshape', extension=hsp_ext,
+            datatype='meg', root=output_path)
         assert op.exists(hsp_fname)
